'use strict';

var context = SP.ClientContext.get_current(); // SharePoint Client Contextを取得する。
var user = context.get_web().get_currentUser(); // SharePointユーザーに関する情報を取得する。
var web; // SharePointサイト情報を格納する。
var listCollection; // リストコレクションを設定・取得する。
var categoryCollection; // カテゴリの一覧を設定・取得する。
var categoryList; // カテゴリの情報を配列で保持する。
var postCollection; // 投稿の一覧を保持する。

(function () {

	// This code runs when the DOM is ready and creates a context object which is 
	// needed to use the SharePoint object model
	$(document).ready(function () {
		getUserName();
		$('#getItems').click(function(event){
			getSharePointSite();
			event.preventDefault();
		});
	});

	// This function prepares, loads, and then executes a SharePoint query to get 
	// the current users information
	function getUserName() {
		context.load(user);
		context.executeQueryAsync(onGetUserNameSuccess, onGetUserNameFail);
	}

	// This function is executed if the above call is successful
	// It replaces the contents of the 'message' element with the user name
	function onGetUserNameSuccess() {
		$('#message').text('Hello ' + user.get_title());
	}

	// This function is executed if the above call fails
	function onGetUserNameFail(sender, args) {
		alert('Failed to get user name. Error: ' + args.get_message());
	}
	
	// SharePointサイトへ接続する。
	function getSharePointSite() {
		var hostWebUrl = decodeURIComponent($('#siteUrl').val());
		
		// 末尾がスラッシュの場合はエラーになるため、末尾のスラッシュを削除する。
		if (hostWebUrl.slice(-1) === '/') {
			hostWebUrl = hostWebUrl.slice(0, -1);
		}
		var parentContext = new SP.AppContextSite(context, hostWebUrl);
		web = parentContext.get_web();
		getListCollection();
	}
	
	// サイトからListのコレクションを取得する。
	function getListCollection() {
		listCollection = web.get_lists();
		listCollection = context.loadQuery(listCollection);
		context.executeQueryAsync(onGetListCollectionSuccess, onGetListCollectionFail);
	}
	
	// リストコレクションの取得に成功したらカテゴリを取得する。
	function onGetListCollectionSuccess() {
		getCategory();
	}
	
	// リストコレクションの取得に失敗した時はエラーを表示する。
	function onGetListCollectionFail(sender, args) {
		alert('Failed to Get List Collection. Error: ' + args.get_message() + '\n' + args.get_stackTrace());
	}
	
	// カテゴリをすべて取得する。
	function getCategory() {		
		for (var i = 0; i < listCollection.length; i++) {
			if (listCollection[i].get_title() === 'カテゴリ') {
				var camlQuery = new SP.CamlQuery.createAllItemsQuery();

				categoryCollection = listCollection[i].getItems(camlQuery);
				
				context.load(categoryCollection);
				context.executeQueryAsync(onGetListCatCollectionSuccess, onGetListItemCollectionFail);
			}
		}
	}
	
	// カテゴリの取得に成功したらカテゴリーの配列を作成する。
	function onGetListCatCollectionSuccess() {
	    var categoryCollectionWithEnumerator = categoryCollection.getEnumerator();
		categoryList = [];
		
		var count = 0;
	    while (categoryCollectionWithEnumerator.moveNext()) {
	        var categoryItems = categoryCollectionWithEnumerator.get_current();
	        var categoryName = categoryItems.get_item('Title');
			var categoryArray = [categoryName, 0];
			categoryList.push(categoryArray);
			count++;
	    }
		var noCategory = ['カテゴリなし',0];
		categoryList.push(noCategory);
		getPost();
	}
	
	// カテゴリの取得に失敗した時はエラーを表示する。
	function onGetListItemCollectionFail(sender, args) {
	    alert('Failed to Get List Item Collection. Error: ' + args.get_message() + '\n' + args.get_stackTrace());
	}
	
	// 投稿をすべて取得する。
	function getPost() {
		for (var i = 0; i < listCollection.length; i++) {
			if (listCollection[i].get_title() === '投稿') {
				var oList = listCollection[i];
				$('#postCount').text(oList.get_itemCount());
				var camlQuery = new SP.CamlQuery.createAllItemsQuery();
				postCollection = listCollection[i].getItems(camlQuery);
				
				context.load(postCollection);
				context.executeQueryAsync(onGetListPostCollectionSuccess, onGetListItemCollectionFail);
				break;
			}
		}
	}
	
	// 投稿の取得に成功後、カテゴリごとに分類し、コメントといいね!の数をカウントしソートする。
	function onGetListPostCollectionSuccess(){
		var postCollectionWithEnumerator = postCollection.getEnumerator();
		
		var postList = [];
		var count = 0;
	    while (postCollectionWithEnumerator.moveNext()) {
			var listItem = postCollectionWithEnumerator.get_current();
	        // 投稿に紐づいたカテゴリを取得する。
			var categoryObject = listItem.get_item('PostCategory');
	        if (categoryObject[0]) {
				for(var i = 0; i < categoryObject.length; i++) {
					var categoryName = categoryObject[i].get_lookupValue();
					for (var j = 0; j < categoryList.length; j++) {
						if(categoryList[j][0] === categoryName) {
							categoryList[j][1]++;
							break;
						}
					}
				}
	        } else {
				categoryList[categoryList.length - 1][1]++;
			}
			// 投稿の一覧とコメント数、いいね数を取得する。
			var postTitle = listItem.get_item('Title');
			var commentCount = listItem.get_item('NumComments');
			var likeCount = listItem.get_item('LikesCount');
			var postArray = ['','',''];
			
			postArray[0] = postTitle;
			postArray[1] = commentCount.get_lookupId();
			if (likeCount) {
				postArray[2] = likeCount;
			} else {
				postArray[2] = 0;
			}
			
			postList.push(postArray);
			count++;
	    }
		$('#categoryTable').empty();
		
		var infoTable = '<table class="rankingTable">';
		infoTable += '<tr>';
		infoTable += '<th>カテゴリ名</th><th>投稿数</th>';
		infoTable += '</tr>';
		
		for(var k = 0; k < categoryList.length; k++) {
			infoTable += '<tr>';
			infoTable += '<td>' + categoryList[k][0] + '</td><td>' + categoryList[k][1] + '</td>';
			infoTable += '</tr>';
		}
		
		infoTable += '</table>';
		$('#categoryTable').append(infoTable);
		
		// 投稿をコメント数で降順ソートする。
		postList.sort(function(a, b) {
			var x = a[1];
			var y = b[1];
			if(x > y) return -1;
			if(y > x) return 1;
			return 0;
		});
		
		var loopCount = 3; // ランキングにて上位3位まで出力する。
		if (loopCount > postList.length) {
			loopCount = postList.length;
		}
		
		// コメント数ランキングを出力する。
		var commentRankTable = '<table class="rankingTable">';
		commentRankTable += '<tr>';
		commentRankTable += '<th>順位</th><th>投稿タイトル</th><th>コメント数</th>';
		commentRankTable += '</tr>';
		
		for(var l = 0; l < loopCount; l++) {
			var rank = l + 1;
			commentRankTable += '<tr>';
			commentRankTable += '<td>' + rank +'</td><td>' + postList[l][0] + '</td><td>' + postList[l][1] + '</td>';
			commentRankTable += '</tr>';
		}
		commentRankTable += '</table>';
		$('#commentRankResult').empty();
		$('#commentRankResult').append(commentRankTable);
		
		// 投稿をいいね！の数で降順ソートする。
		postList.sort(function(a, b) {
			var x = a[2];
			var y = b[2];
			if(x > y) return -1;
			if(y > x) return 1;
			return 0;
		});
		
		// いいね！ランキングを出力する。　
		var likesTable = '<table class="rankingTable">';
		likesTable += '<tr>';
		likesTable += '<th>順位</th><th>投稿タイトル</th><th>いいね！の数</th>';
		likesTable += '</tr>';
		for(var m = 0; m < loopCount; m++) {
			var likesRank = m + 1;
			likesTable += '<tr>';
			likesTable += '<td>'+ likesRank + '</td><td>' + postList[m][0] + '</td><td>' + postList[m][2] + '</td>';
			likesTable += '</tr>';
		}
		
		likesTable += '</table>';
		$('#likesRankResult').empty();
		$('#likesRankResult').append(likesTable);
	}
})();